﻿using System;

namespace Falcon.SugarApi.DatabaseManager
{
    /// <summary>
    /// 允许对字段进行加密存储
    /// </summary>
    [AttributeUsage(AttributeTargets.Property,AllowMultiple = false,Inherited = false)]
    public class EnableEncryptAttribute:Attribute
    {
        /// <summary>
        /// 用于加密的类型
        /// </summary>
        public Type EncryptType { get; set; }

        /// <summary>
        /// 通过提供加密解密接口实现字段加密
        /// </summary>
        /// <param name="encryptType">实现加密解密的类型</param>
        /// <exception cref="ArgumentException">提供的类型不符合要求</exception>
        public EnableEncryptAttribute(Type encryptType) {
            if(!typeof(IEncrypt).IsAssignableFrom(encryptType)) {
                throw new ArgumentException("encryptType must implement IEncrypt");
            }
            this.EncryptType = encryptType;
        }
    }
}
