﻿using System.Threading;
using System.Threading.Tasks;

namespace Falcon.SugarApi.TimedTask.TaskManager
{
    /// <summary>
    /// 任务管理接口
    /// </summary>
    public interface ITaskManager
    {
        /// <summary>
        /// 任务名称
        /// </summary>
        string TaskName { get; }
        /// <summary>
        /// 任务说明
        /// </summary>
        string TaskDesc { get; }
        /// <summary>
        /// 任务计划
        /// </summary>
        string TaskSchedule { get; }
        /// <summary>
        /// 任务分组
        /// </summary>
        string? TaskGroup { get; }

        /// <summary>
        /// 运行一次任务
        /// </summary>
        /// <param name="cancellationToken">退出信号</param>
        /// <returns>控制结果</returns>
        Task<TaskControlResult> RunOnceAsync(CancellationToken cancellationToken = default);
        /// <summary>
        /// 开始任务
        /// </summary>
        /// <param name="cancellationToken">退出信号</param>
        /// <returns>控制结果</returns>
        Task<TaskControlResult> StartTaskAsync(CancellationToken cancellationToken = default);
        /// <summary>
        /// 停止任务
        /// </summary>
        /// <param name="cancellationToken">退出信号</param>
        /// <returns>控制结果</returns>
        Task<TaskControlResult> StopTaskAsync(CancellationToken cancellationToken = default);
        /// <summary>
        /// 获得当前任务状态
        /// </summary>
        /// <returns>当前任务状态</returns>
        Task<TaskState> GetTaskStateAsync();
        /// <summary>
        /// 获取上次运行信息
        /// </summary>
        /// <returns>结果信息</returns>
        Task<LastRunInformation> GetLastRunInfomationAsync();
    }
}
